//------------------------------------------------------------------
#property copyright "mladen"
#property link      "mladenfx@gmail.com"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  clrDodgerBlue
#property indicator_color2  clrSandyBrown
#property indicator_width1  2
#property indicator_width2  2
#property indicator_minimum 0
#property indicator_maximum 1
#property strict

//
//
//
//
//

enum colorOn
{
   clrOnSlope, //Color change on slope change
   clrOnZero,  // Color change on zero cross
   clrOnSign   // Color change on signal cross
};
extern int    FastMAPeriod = 23;  // Fast macd period
extern int    SlowMAPeriod = 50;  // Slow macd period
extern int    SignalPeriod = 9;   // Signal period
extern int    StdPeriod    = 50;  // Standard deviation used for normalization
extern ENUM_APPLIED_PRICE Price         = PRICE_CLOSE; // Price to use 
extern double Filter       = 0;   // Filter to apply to macd value
extern int    FilterPeriod = 0;   // Filter period to use (<= to use the slow macd period)
extern colorOn            ColorChangeOn = clrOnSlope;  // Color change on : 

//
//
//
//
//

double macdBuffer[];
double macdBufferhu[];
double macdBufferhd[];
double signal[];
double slope[];
double prices[];


//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
      SetIndexBuffer(0,macdBufferhu); SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexBuffer(1,macdBufferhd); SetIndexStyle(1,DRAW_HISTOGRAM);
      SetIndexBuffer(2,macdBuffer);
      SetIndexBuffer(3,signal);
      SetIndexBuffer(4,slope);
      SetIndexBuffer(5,prices);
   IndicatorShortName("MACD (std normalized) ("+(string)FastMAPeriod+","+(string)SlowMAPeriod+","+(string)Filter+","+(string)FilterPeriod+")");
   return(0);
}

int deinit()
{
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars < 0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);

   //
   //
   //
   //
   //

   int fltPeriod = FilterPeriod; if (fltPeriod<=0) fltPeriod = SlowMAPeriod;   
   for(int i = limit; i >= 0; i--)
   {
      prices[i] = iMA(NULL,0,1,0,MODE_SMA,Price,i);
      double std = iStdDev(NULL,0,StdPeriod,0,MODE_SMA,Price,i);
         if (std!=0)
               macdBuffer[i] = iFilter((iEma(prices[i],FastMAPeriod,i,0)-iEma(prices[i],SlowMAPeriod,i,1))/std,Filter,fltPeriod,i);
         else  macdBuffer[i] = iFilter(                                                                      0,Filter,fltPeriod,i);
         signal[i] = iEma(macdBuffer[i],SignalPeriod,i,2);
      
      if (i>=Bars-1) continue;
      
      //
      //
      //
      //
      //
      
      macdBufferhu[i] = EMPTY_VALUE;
      macdBufferhd[i] = EMPTY_VALUE;
      slope[i]       = slope[i+1];
      switch (ColorChangeOn)
      {
         case clrOnSlope: 
            if (macdBuffer[i] > macdBuffer[i+1]) slope[i] =  1;
            if (macdBuffer[i] < macdBuffer[i+1]) slope[i] = -1;
            break;
         case clrOnZero: 
            if (macdBuffer[i] > 0) slope[i] =  1;
            if (macdBuffer[i] < 0) slope[i] = -1;
            break;
         default : 
            if (macdBuffer[i] > signal[i]) slope[i] =  1;
            if (macdBuffer[i] < signal[i]) slope[i] = -1;
            break;
      }      
      if (slope[i]== 1) macdBufferhu[i] = 1;
      if (slope[i]==-1) macdBufferhd[i] = 1;
   }   
   return(0);
}


//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

double workEma[][3];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r=Bars-r-1;
   if (period<=1) { workEma[r][instanceNo]=price; return(price); }

   //
   //
   //
   //
   //
      
   workEma[r][instanceNo] = price;
   double alpha = 2.0 / (1.0+period);
   if (r>0)
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//
//
//
//
//

#define filterInstances 1
double workFil[][filterInstances*3];

#define _fchange 0
#define _fachang 1
#define _fprice  2

double iFilter(double tprice, double filter, int period, int i, int instanceNo=0)
{
   if (filter<=0) return(tprice);
   if (ArrayRange(workFil,0)!= Bars) ArrayResize(workFil,Bars); i = Bars-i-1; instanceNo*=3;
   
   //
   //
   //
   //
   //
   
   workFil[i][instanceNo+_fprice]  = tprice; if (i<1) return(tprice);
   workFil[i][instanceNo+_fchange] = MathAbs(workFil[i][instanceNo+_fprice]-workFil[i-1][instanceNo+_fprice]);
   workFil[i][instanceNo+_fachang] = workFil[i][instanceNo+_fchange];

   for (int k=1; k<period && (i-k)>=0; k++) workFil[i][instanceNo+_fachang] += workFil[i-k][instanceNo+_fchange];
                                            workFil[i][instanceNo+_fachang] /= period;
    
   double stddev = 0; for (int k=0;  k<period && (i-k)>=0; k++) stddev += MathPow(workFil[i-k][instanceNo+_fchange]-workFil[i-k][instanceNo+_fachang],2);
          stddev = MathSqrt(stddev/(double)period); 
   double filtev = filter * stddev;
   if( MathAbs(workFil[i][instanceNo+_fprice]-workFil[i-1][instanceNo+_fprice]) < filtev ) workFil[i][instanceNo+_fprice]=workFil[i-1][instanceNo+_fprice];
        return(workFil[i][instanceNo+_fprice]);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}